import React, { useState, useMemo, useEffect } from 'react';
import { File, Folder, ChevronRight, ChevronDown, Loader2 } from 'lucide-react';
import { useMenuStore, init } from './store/menu.ts';
import type { MenuItem } from './store/menu.ts';

// 辅助函数：获取目录的直接子项
const getDirectChildren = (path: string, menu: MenuItem[]): MenuItem[] => {
  const pathPrefix = path ? path + '/' : '';

  return menu.filter((item) => {
    if (!item.path.startsWith(pathPrefix)) return false;
    if (item.path === path) return false;

    const relativePath = item.path.slice(pathPrefix.length);
    return !relativePath.includes('/');
  });
};

interface DirectoryItemProps {
  item: MenuItem;
  level: number;
  prefix?: string;
}

const DirectoryItem: React.FC<DirectoryItemProps> = ({ item, level, prefix }) => {
  const [expanded, setExpanded] = useState(false);
  const { menu } = useMenuStore();
  const children = useMemo(() => {
    const children = getDirectChildren(item.path, menu);
    return children.sort((a, b) => {
      if (a.type === 'directory' && b.type === 'file') return -1; // 目录排在前面
      if (a.type === 'file' && b.type === 'directory') return 1; // 文件排在后面
      return a.path.localeCompare(b.path); // 同类型按路径排序
    });
  }, [menu, item.path]);
  const directory = item.path.replace(prefix || '', '').replace(/\/$/, '') || '/'; // 去掉前缀和末尾斜杠
  const newPrefix = item.path + '/';
  return (
    <div>
      <div
        className='flex items-center cursor-pointer py-1 px-2 hover:bg-gray-100 dark:hover:bg-gray-800 rounded group'
        style={{ paddingLeft: `${level * 12}px` }}
        onClick={() => setExpanded(!expanded)}>
        {children.length > 0 ? (
          expanded ? (
            <ChevronDown className='w-4 h-4 mr-1 text-gray-500' />
          ) : (
            <ChevronRight className='w-4 h-4 mr-1 text-gray-500' />
          )
        ) : (
          <div className='w-4 h-4 mr-1' />
        )}
        <Folder className='w-4 h-4 mr-2 text-yellow-500 shrink-0' />
        <span className='text-sm truncate'>{directory}</span>
      </div>

      {expanded && children.length > 0 && (
        <div className='ml-2'>
          {children.map((child) =>
            child.type === 'directory' ? (
              <DirectoryItem key={child.path} item={child} prefix={newPrefix} level={level + 1} />
            ) : (
              <FileItem key={child.path} item={child} prefix={newPrefix} level={level + 1} />
            ),
          )}
        </div>
      )}
    </div>
  );
};

interface FileItemProps {
  item: MenuItem;
  level: number;
  prefix?: string;
}

const FileItem: React.FC<FileItemProps> = ({ item, level, prefix }) => {
  const { currentPath, setCurrentPath } = useMenuStore();

  return (
    <div
      className={`flex items-center cursor-pointer py-1 px-2 hover:bg-gray-100 dark:hover:bg-gray-800 rounded ${
        currentPath === item.path ? 'bg-blue-100 dark:bg-blue-900' : ''
      }`}
      style={{ paddingLeft: `${level * 12}px` }}
      onClick={() => setCurrentPath(item.path)}>
      <div className='w-4 h-4 mr-1' /> {/* 占位，保持对齐 */}
      <File className='w-4 h-4 mr-2 text-blue-500 shrink-0' />
      <span className='text-sm truncate'>{item.path.replace(prefix || '', '')}</span>
    </div>
  );
};

export const Sidebar: React.FC = () => {
  const { menu, isLoading } = useMenuStore();
  useEffect(() => {
    // 初始化菜单数据
    init();
  }, []);
  // 获取顶层项（根目录下的文件和文件夹）
  const rootItems = useMemo(() => {
    // return menu.filter((item) => !item.path.includes('/'));
    const _menu = menu.filter((item) => {
      return !item.path.includes('/');
    });
    return _menu.sort((a, b) => {
      if (a.type === 'directory' && b.type === 'file') return -1; // 目录排在前面
      if (a.type === 'file' && b.type === 'directory') return 1; // 文件排在后面
      return a.path.localeCompare(b.path); // 同类型按路径排序
    });
  }, [menu]);

  return (
    <div className='border-r w-full border-gray-200 dark:border-gray-700 min-h-screen p-2 bg-white dark:bg-gray-900 h-full'>
      <div className='flex items-center justify-between mb-4 px-2'>
        {/* <h2 className='text-lg font-semibold'>文件</h2> */}
        {isLoading && <Loader2 className='w-4 h-4 animate-spin text-gray-500' />}
      </div>
      <div className='overflow-y-auto scrollbar' style={{ maxHeight: 'calc(100% - 58px)' }}>
        {menu.length === 0 && !isLoading ? (
          <div className='text-sm text-gray-500 px-2'>暂无文件</div>
        ) : (
          <div className='space-y-1'>
            {rootItems.map((item) =>
              item.type === 'directory' ? <DirectoryItem key={item.path} item={item} level={1} /> : <FileItem key={item.path} item={item} level={1} />,
            )}
          </div>
        )}
      </div>
    </div>
  );
};

export default Sidebar;
