import { create } from 'zustand';
import { query } from '@/modules/query';
import { QueryLoginBrowser } from '@/query/query-login/query-login-browser';
import { QueryResources } from '@/query/query-resources/index';

export const queryLogin = new QueryLoginBrowser({ query });
export const queryResources = new QueryResources({
  prefix: '/root/resources/',
});

export type MenuItem = {
  type: 'file' | 'directory';
  name?: string; // type === 'file'
  prefix?: string; // type === 'directory'
  etag?: string;
  lastModified?: string; // ISO date string
  size: number;
  url: string;
  path: string; // a/b/file.md
  pathname: string;
};

export type Menu = MenuItem[];

interface MenuState {
  menu: Menu;
  currentPath: string | null;
  isLoading: boolean;
  setMenu: (menu: Menu) => void;
  setCurrentPath: (path: string | null) => void;
  setLoading: (status: boolean) => void;
}

export const useMenuStore = create<MenuState>((set) => ({
  menu: [],
  currentPath: null,
  isLoading: false,
  setMenu: (menu) => set({ menu }),
  setCurrentPath: (currentPath) => set({ currentPath }),
  setLoading: (isLoading) => set({ isLoading }),
}));

class Status {
  isInitialized = false;
  username = '';
}
const status = new Status();

export const init = async (prefix: string = '') => {
  const { setMenu, setCurrentPath, setLoading } = useMenuStore.getState();
  let me = await queryLogin.checkLocalUser();
  const isInitialized = status.isInitialized;
  if (!isInitialized && me) {
    status.isInitialized = true;
    status.username = me.username!;
    queryResources.setUsername(status.username);
  }
  let recursive = true;
  const data = {};
  if (recursive) {
    data['recursive'] = recursive;
  }
  const res = await queryResources.getList(prefix, data);
  if (res.code === 200) {
    const menu = res.data!.map((item: any) => {
      if (item.prefix) {
        item.type = 'directory';
      } else {
        item.type = 'file';
      }
      return item;
    });
    console.log('init menu', menu);
    if (recursive) {
      const obj: Record<string, any> = {};
      menu.forEach((item) => {
        const parts = item.path.split('/');
        const dirParts = parts.slice(0, -1);
        for (let i = 0; i < dirParts.length; i++) {
          const dir = dirParts.slice(0, i + 1).join('/');
          if (!dir) continue; // skip root
          obj[dir] = obj[dir] || { type: 'directory', path: dir };
        }
      });
      Object.keys(obj).forEach((key) => {
        const item = obj[key];
        menu.push(item);
      });
    }
    setMenu(menu);
    setCurrentPath('');
  }
};

export const getCurrentContent = async (currentPath: string): Promise<string | null> => {
  if (!currentPath) return null;
  const res = await queryResources.fetchFile(currentPath);
  if (res.success) {
    return res.data;
  } else {
    console.error('Error fetching content:', res.message);
    return null;
  }
};
