import { createEditor } from '@kevisual/codemirror';
import { Chain } from '@kevisual/codemirror/utils';
import { IconButton } from '@/components/a/button.tsx';
import { Tooltip } from '@/components/a/tooltip.tsx';
import { UploadIcon, UploadCloud as CloudUploadOutlined } from 'lucide-react';
import { useEffect, useRef, useState } from 'react';
import { useDropzone } from 'react-dropzone';
import { CacheWorkspace } from '@kevisual/cache';
import { useHomeStore } from './store/index.ts';
import { UploadModal } from './module/UploadModal.tsx';
import { useShallow } from 'zustand/shallow';
import { toast } from 'react-toastify';
import { SuccessModal } from './module/SuccessModal.tsx';
import { debounce } from 'lodash-es';
const chain = new Chain();
export const Home = () => {
  const editorElRef = useRef<HTMLDivElement>(null);
  const editorRef = useRef<ReturnType<typeof createEditor>>(null);
  const [isHtml, setIsHtml] = useState(true);
  const [html, setHtml] = useState('');
  const { initApp, setOpenUploadModal, setText, filename, openPreview, setOpenPreview } = useHomeStore(
    useShallow((state) => ({
      initApp: state.initApp,
      setOpenUploadModal: state.setOpenUploadModal,
      setText: state.setText,
      filename: state.filename,
      openPreview: state.openPreview,
      setOpenPreview: state.setOpenPreview,
    })),
  );
  const onDrop = (acceptedFiles) => {
    console.log(acceptedFiles);
    const file = acceptedFiles[0];
    const reader = new FileReader();
    reader.onload = (e) => {
      const content = e.target?.result as string;
      if (editorRef.current) {
        editorRef.current.dispatch({
          changes: { from: 0, to: editorRef.current.state.doc.length, insert: content },
        });
      } else {
        toast.error('编辑器未初始化，请稍后再试');
      }
    };
    reader.readAsText(file);
  };
  const { getRootProps, getInputProps } = useDropzone({ onDrop, accept: { 'text/html': ['.html'], 'text/javascript': ['.js'], 'text/css': ['.css'] } });

  useEffect(() => {
    initApp();
    initEditor();
    return () => {
      if (editorRef.current) {
        chain.destroy();
      }
    };
  }, []);

  const initEditor = async () => {
    if (!editorElRef.current) return;
    const cache = new CacheWorkspace();
    let cacheData = '';
    try {
      cacheData = (await cache.storage.get('html-editor')) || '';
    } catch (error) {
      console.error(error);
    }
    const type = filename.endsWith('.js') ? 'javascript' : filename.endsWith('.css') ? 'css' : 'html';
    setIsHtml(type === 'html');
    const debouncedSetHtml = debounce((value: string) => {
      setHtml(value);
    }, 500);
    const editor = createEditor(editorElRef.current, {
      type: type || 'html',
      onChange: (value) => {
        cache.storage.set('html-editor', value);
        debouncedSetHtml(value);
      },
    });
    const cmScroller = editorElRef.current.querySelector('.cm-scroller');
    if (cmScroller) {
      cmScroller.classList.add('scrollbar');
    }
    chain.setEditor(editor);
    editorRef.current = editor;
    setTimeout(() => {
      if (cacheData) {
        editorRef.current!.dispatch({
          changes: { from: 0, to: editorRef.current!.state.doc.length, insert: cacheData },
        });
      }
    }, 300);
  };

  return (
    <div className='flex flex-row w-full h-full'>
      <div className='flex flex-col text-primary border-r border-gray-200 px-2 pt-2'>
        <div>
          <Tooltip title='部署应用' placement='right'>
            <IconButton
              onClick={() => {
                // const editorContent = editorRef.current?.getContent();
                const editorContent = editorRef.current?.state.doc.toString();
                if (editorContent) {
                  setOpenUploadModal(true);
                  setText(editorContent);
                } else {
                  toast.error('请先输入代码');
                }
              }}>
              <CloudUploadOutlined />
            </IconButton>
          </Tooltip>
        </div>
        <div className=' mt-2' {...getRootProps()}>
          <Tooltip title='点击上传html或者js文件  ' placement='right'>
            <div>
              <IconButton>
                <UploadIcon size={16} />
              </IconButton>
              <input type='file' style={{ display: 'none' }} {...getInputProps()} />
            </div>
          </Tooltip>
        </div>
      </div>
      <div className='h-full grow px-2 flex flex-col'>
        <div className=' py-1'>
          <i className=' text-gray-500' style={{ fontSize: 10 }}>
            {'>'} 快速部署html小应用, 粘贴前端html代码。点击部署。(这个页面内容自动缓存到本地)
            <a className='text-blue-500 ml-2' href='/root/center/app/edit/list'>
              管理
            </a>
          </i>
        </div>
        <div className='w-full flex overflow-hidden' style={{ height: 'calc(100% - 50px)' }}>
          <div className='w-full h-full border rounded-md border-gray-200 scrollbar relative'>
            <div className='h-full overflow-hidden' ref={editorElRef}></div>
            {isHtml && (
              <IconButton className='absolute top-2 right-2'>
                <Tooltip title='预览' placement='right'>
                  <input
                    className='cursor-pointer'
                    type='checkbox'
                    checked={openPreview}
                    onChange={(e) => {
                      setOpenPreview(e.target.checked);
                    }}
                  />
                </Tooltip>
              </IconButton>
            )}
          </div>
          {isHtml && openPreview && (
            <div className='w-1/2 h-full shrink-0'>
              <iframe
                className='w-full h-full border rounded-md border-gray-200 scrollbar'
                srcDoc={html}
                title='Preview'
                sandbox='allow-scripts allow-same-origin'
                style={{ width: '100%', height: '100%' }}></iframe>
            </div>
          )}
        </div>
      </div>
      <UploadModal />
      <SuccessModal />
    </div>
  );
};
